@@ -138,8 +138,7 @@ def test_get_successful_authenticated():
             assert isinstance(response.content, bytes)
 
 
-# providing just the api_key
-def test_get_successful_as_str():
+def test_get_with_header_array_in_options():
     api_key = fake.pystr()
 
     format = random.choice(
@@ -152,37 +151,41 @@ def test_get_successful_as_str():
         "format": format,
         "full_page": random.choice([True, False]),
         "width": fake.random_int(),
+        "header": [
+            "x-my-first-header=somevalue",
+            "x-my-second-header=someothervalue",
+        ],
     }
 
+    urlbox_request_url = (
+        f"{UrlboxClient.BASE_API_URL}"
+        f"{api_key}/{format}"
+        f"?{urllib.parse.urlencode(options, doseq=True)}"
+    )
+
     urlbox_client = UrlboxClient(api_key=api_key)
 
-    response = urlbox_client.get(options, to_string=True)
+    with requests_mock.Mocker() as requests_mocker:
+        with open(
+            "tests/files/urlbox_screenshot.png", "rb"
+        ) as urlbox_screenshot:
+            requests_mocker.get(
+                urlbox_request_url,
+                content=urlbox_screenshot.read(),
+                headers={"content-type": f"image/{format}"},
+            )
 
-    assert isinstance(response, str)
+            response = urlbox_client.get(options)
 
+            assert response.status_code == 200
+            assert format in response.headers["Content-Type"]
+            assert isinstance(response, requests.models.Response)
+            assert isinstance(response.content, bytes)
 
-def test_get_successful_as_str_with_api_secret():
-    api_key = fake.pystr()
-    api_secret = fake.pystr()
-
-    format = random.choice(
-        ["png", "jpg", "jpeg", "avif", "webp", "pdf", "svg", "html"]
-    )
-    url = fake.url()
-
-    options = {
-        "url": url,
-        "format": format,
-        "full_page": random.choice([True, False]),
-        "width": fake.random_int(),
-    }
-
-    urlbox_client = UrlboxClient(api_key=api_key, api_secret=api_secret)
-    response = urlbox_client.get(options, to_string=True)
-
-    assert isinstance(response, str)
-    # It still returns the unautenticated get url
-    assert api_secret not in response
+            assert (
+                "header=x-my-first-header%3Dsomevalue&header=x-my-second-header%3Dsomeothervalue"
+                in urlbox_request_url
+            )
 
 
 # valid url but with white spaces before and after
@@ -701,3 +704,49 @@ def test_post_request_unsuccessful_missing_api_secret():
         in str(ex.value)
     )
 
+
+# Test generate_url
+def test_generate_url_without_api_secret():
+    api_key = fake.pystr()
+
+    format = random.choice(
+        ["png", "jpg", "jpeg", "avif", "webp", "pdf", "svg", "html"]
+    )
+    url = fake.url()
+
+    options = {
+        "url": url,
+        "format": format,
+        "full_page": random.choice([True, False]),
+        "width": fake.random_int(),
+    }
+
+    urlbox_client = UrlboxClient(api_key=api_key)
+
+    urlbox_url = urlbox_client.generate_url(options)
+
+    assert isinstance(urlbox_url, str)
+
+
+def test_get_successful_as_str_with_api_secret():
+    api_key = fake.pystr()
+    api_secret = fake.pystr()
+
+    format = random.choice(
+        ["png", "jpg", "jpeg", "avif", "webp", "pdf", "svg", "html"]
+    )
+    url = fake.url()
+
+    options = {
+        "url": url,
+        "format": format,
+        "full_page": random.choice([True, False]),
+        "width": fake.random_int(),
+    }
+
+    urlbox_client = UrlboxClient(api_key=api_key, api_secret=api_secret)
+    urlbox_url = urlbox_client.generate_url(options)
+
+    assert isinstance(urlbox_url, str)
+    # It doesn't leak the api_secret (uses the tokenised options instead)
+    assert api_secret not in urlbox_url
